{
  description = "sapling";

  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs";
    flake-utils.url = "github:numtide/flake-utils";
  };

  outputs = { self, nixpkgs, flake-utils }:
    flake-utils.lib.eachDefaultSystem (system:
      let
        pkgs              = nixpkgs.legacyPackages.${system};
        packageName       = "sapling";
        containerPackageName = "${packageName}-container";

        customGHC = pkgs.haskellPackages.ghcWithPackages (hpkgs: with hpkgs; [
          megaparsec
        ]);

        haskellPackages = pkgs.haskellPackages;

        enableSharedExecutables = false;
        enableSharedLibraries = false;

        sapling = pkgs.haskell.lib.justStaticExecutables self.packages.${system}.default;
      in {

        packages.${packageName} =
          haskellPackages.callCabal2nix packageName self rec {};

        packages.default = self.packages.${system}.${packageName};
        defaultPackage = self.packages.${system}.default;

        devShells.default = pkgs.mkShell {
          buildInputs = with pkgs.haskellPackages; [
            cabal-install
            ghcid
            customGHC
          ];
          inputsFrom = builtins.attrValues self.packages.${system};
        };
        devShell = self.devShells.${system}.default;

        packages.${containerPackageName} = pkgs.dockerTools.buildImage {
          name = "sapling";

          copyToRoot = pkgs.buildEnv {
            name = "image-root";
            paths = [ sapling ];
            pathsToLink = [ "/bin" ];
          };
          tag = "latest";
          config = {
            Cmd = [
              "/bin/sapling"
            ];
            WorkingDir = "/app";
            ExposedPorts = {
              "3000/tcp" = {};
            };
            extraCommands = ''
            '';
          };
        };
      });
}
